/*====================================================== file = p1.b =====*/
/*= BOOL program to generate logic for PP-VOQ input port block 1 of 3    =*/
/*=   - Rotate (A) by H_last amount                                      =*/
/*========================================================================*/
/*= NOTES: 1) MSb is high-numbered bit                                   =*/
/*=        2) A is (T_in AND Q) from previous AND stage                  =*/
/*=----------------------------------------------------------------------=*/
/*=  Build and execute: bool p1.b > outfile                              =*/
/*=----------------------------------------------------------------------=*/
/*=  Author: Kenneth J. Christensen                                      =*/
/*=          University of South Florida                                 =*/
/*=          WWW: http://www.csee.usf.edu/~christen                      =*/
/*=          Email: christen@csee.usf.edu                                =*/
/*=----------------------------------------------------------------------=*/
/*=  History: KJC (06/22/00) - Genesis                                   =*/
/*========================================================================*/

/***** Definitions *****/
/* N is number of ports, M = ceil(log2(N))                                */
#define N    4
#define M    2

/***** Declarations *****/
input   A[N];       /* Vector - this is T_in AND Q                        */
input   H_last[M];  /* Value - mask-out value                             */
output  AR[N];      /* Vector - rotated A vector                          */
boolean Temp[N];    /* Vector - (Q AND T_in)                              */
integer i, j, k;    /* Loop counters                                      */
integer x;          /* Temporary value for simulation                     */

/***** Behavior description *****/
for (i = 0, (N-1))
  if (H_last == @i)
  {
    j = i;
    for (k = 0, (N-1))
    {
      AR[k] = A[j];
      j = j + 1;
      if (j == N) j = 0;
    }
  }

/***** Minimize *****/
lminimize(10, AR);

/***** Output results *****/
print(AR);

/**** Simulate *****/
x = 1;
for (i = 1, N)
  x = x * 2;

values(Q = @(x-1));
for (i = 0, (x-1))
  for (j = 0, (N-1))
  {
    values(A = @i);
    values(H_last = @j);
    printf(1,"A = %b  H_last = %b  AR = %b \n", A, H_last, AR);
  }

end;
