/*====================================================== file = p2.b =====*/
/*= BOOL program to generate logic for PP-VOQ input port block 2 of 3    =*/
/*=   - Scan rotated (Q AND T_in) vector for next "1" bit                =*/
/*========================================================================*/
/*= NOTES: 1) MSb is high-numbered bit                                   =*/
/*=        2) The overflow of "H_next := H_last + @i" is intentional     =*/
/*=           and correct.                                               =*/
/*=----------------------------------------------------------------------=*/
/*=  Build and execute: bool p2.b > outfile                              =*/
/*=----------------------------------------------------------------------=*/
/*=  Author: Kenneth J. Christensen                                      =*/
/*=          University of South Florida                                 =*/
/*=          WWW: http://www.csee.usf.edu/~christen                      =*/
/*=          Email: christen@csee.usf.edu                                =*/
/*=----------------------------------------------------------------------=*/
/*=  History: KJC (06/22/00) - Genesis                                   =*/
/*========================================================================*/

/***** Definitions *****/
/* N is number of ports, M = ceil(log2(N))                                */
#define N    4
#define M    2

/***** Declarations *****/
input   AR[N];      /* Vector - Rotated A                                 */
input   H_last[M];  /* Value - mask-in value                              */
output  H_next[M];  /* Value - mask-out value                             */
integer i, j;       /* Loop counters                                      */
integer x;          /* Temporary value for simulation                     */

/***** Behavior description *****/
for (i = 1, (N-1))
  if (AR[i] == 1)
  {
    H_next := H_last + @i;
    break;
  }
if (AR == @0) H_next = H_last;

/***** Minimize *****/
lminimize(10, H_next);

/***** Output results *****/
print(H_next);

/**** Simulate *****/
x = 1;
for (i = 1, N)
  x = x * 2;

for (i = 0, (x-1))
  for (j = 0, (N-1))
  {
    values(AR = @i);
    values(H_last = @j);
    printf(1,"AR = %b  H_last = %b  H_next = %b \n", AR, H_last, H_next);
  }

end;
