/*====================================================== file = p3.b =====*/
/*= BOOL program to generate logic for PP-VOQ input port block 3 of 3    =*/
/*=   - Mask block                                                       =*/
/*========================================================================*/
/*= NOTES: 1) MSb is high-numbered bit                                   =*/
/*=        2) In the case of H_last == H_next, H_next shoud prevail      =*/
/*=           (i.e., the token should be masked-out since it is being    =*/
/*=           held for the next forwarding phase).                       =*/
/*=----------------------------------------------------------------------=*/
/*=  Build and execute: bool p3.b > outfile                              =*/
/*=----------------------------------------------------------------------=*/
/*=  Author: Kenneth J. Christensen                                      =*/
/*=          University of South Florida                                 =*/
/*=          WWW: http://www.csee.usf.edu/~christen                      =*/
/*=          Email: christen@csee.usf.edu                                =*/
/*=----------------------------------------------------------------------=*/
/*=  History: KJC (06/22/00) - Genesis                                   =*/
/*========================================================================*/

/***** Definitions *****/
/* N is number of ports, M = ceil(log2(N))                                */
#define N   4
#define M   2

/***** Declarations *****/
input   T_in[N];    /* Vector - tokens in                                 */
input   H_last[M];  /* Value - mask-in value                              */
input   H_next[M];  /* Value - mask-out value                             */
output  T_out[N];   /* Vector - tokens out                                */
boolean M_last[N];  /* Vector - mask for H_last                           */
boolean M_next[N];  /* Vector - mask for H_next                           */
boolean Temp[N];    /* Vector - temporary T_out                           */
integer i, j;       /* Loop counters                                      */
integer x;          /* Temporary value for simulation                     */

/***** Behavior description *****/
for (i = 0, (N-1))
  for (j = 0, (N-1))
    if ((H_last == @i) & (H_next == @j))
    {
      M_last := @1;
      M_next := @1;
      M_last := M_last << i;
      M_next := M_next << j;
      Temp := T_in | M_last;
      Temp := Temp & (~M_next);
      T_out := Temp;
    }

/***** Minimize *****/
lminimize(10, T_out);

/***** Output results *****/
print(T_out);

/**** Simulate *****/
x = 1;
for (i = 1, N)
  x = x * 2;

values(T_in = @0);
for (i = 0, (N-1))
  for (j = 0, (N-1))
  {
    values(H_last = @i);
    values(H_next = @j);
    printf(1,"T_in = %b  H_last = %b  H_next = %b  T_out = %b \n",
      T_in, H_last, H_next, T_out);
  }
values(T_in = @(x-1));
for (i = 0, (N-1))
  for (j = 0, (N-1))
  {
    values(H_last = @i);
    values(H_next = @j);
    printf(1,"T_in = %b  H_last = %b  H_next = %b  T_out = %b \n",
      T_in, H_last, H_next, T_out);
  }

end;
